/*
 * This file is part of GPaste.
 *
 * Copyright (c) 2010-2022, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
 */

#pragma once

#define __G_PASTE_GTK3_H_INSIDE__

/* Misc. macros */
#include <gpaste-gtk3/gpaste-gtk-macros.h>
#include <gpaste-gtk3/gpaste-gtk-util.h>

/* GPasteGtkSettingUi */
#include <gpaste-gtk3/gpaste-gtk-settings-ui-panel.h>
#include <gpaste-gtk3/gpaste-gtk-settings-ui-stack.h>
#include <gpaste-gtk3/gpaste-gtk-settings-ui-widget.h>

#undef __G_PASTE_GTK3_H_INSIDE__
